<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\Icons;
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\Icon;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\Icons as IconsRepository;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\{Anchor, Id as IconId};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;
use Locr\WordPress\Plugin\LocrMaps\{Plugin, TypedSequence};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icon list table.
 */
final class IconListTable extends \WP_List_Table {
	/**
	 * This function displays the checkbox column.
	 *
	 * @param Icon $item The item.
	 */
	public function column_cb( $item ): string {
		return sprintf(
			'<input type="checkbox" name="id[]" value="%s" />',
			$item->id->value
		);
	}

	/**
	 * This function displays the table.
	 *
	 * @param Icon   $item The item.
	 * @param string $column_name The column name.
	 * @return mixed
	 */
	public function column_default( $item, $column_name ): mixed {
		return match ( $column_name ) {
			'id' => $item->id->value,
			'icon_url' => '<img src="' . esc_attr( (string) $item->url ) . '" alt="preview" style="max-width: 36px; max-height: 36px;" />',
			'name' => (string) $item->name,
			'scale' => ( $item->scale->value * 100 ) . '%',
			'size' => $item->size->width . 'x' . $item->size->height . 'px',
			'anchor' => $item->anchor->value,
			default => null
		};
	}

	/**
	 * This function displays the ID column.
	 *
	 * @param Icon $item The item.
	 */
	protected function column_id( $item ): string {
		$admin_page_url = admin_url( 'admin.php' );

		$icon_id = $item->id->value;

		// row action to delete the icon item.
		$page = filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'page', FILTER_DEFAULT );

		$actions                   = array();
		$query_args_edit_icon_item = array(
			'page'     => $page,
			'action'   => 'edit_icon_item',
			'icon_id'  => $icon_id,
			'_wpnonce' => wp_create_nonce( 'edit_icon_item' ),
		);
		$edit_icon_item_url        = esc_url( add_query_arg( $query_args_edit_icon_item, $admin_page_url ) );
		$actions['edit']           = sprintf( '<a href="%s">%s</a>', $edit_icon_item_url, __( 'Edit', 'default' ) );

		// row action to delete the icon item.
		$query_args_delete_icon_item = array(
			'page'     => $page,
			'action'   => 'delete_icon_item',
			'icon_id'  => $icon_id,
			'_wpnonce' => wp_create_nonce( 'delete_icon_item' ),
		);
		$delete_icon_item_url        = esc_url( add_query_arg( $query_args_delete_icon_item, $admin_page_url ) );
		$actions['trash']            = sprintf( '<a href="%s">%s</a>', $delete_icon_item_url, __( 'Delete', 'default' ) );

		return sprintf( '%1$s %2$s', $icon_id, $this->row_actions( $actions ) );
	}

	/**
	 * This function displays the icon edit form.
	 *
	 * @param int|string $id The id.
	 */
	public function display_edit_form( $id ): void {
		$request_method = filter_input( INPUT_SERVER, 'REQUEST_METHOD', FILTER_DEFAULT ) ?? '';
		if ( empty( $request_method ) && isset( $_SERVER['REQUEST_METHOD'] ) ) {
			$request_method = filter_var( wp_unslash( $_SERVER['REQUEST_METHOD'] ), FILTER_DEFAULT );
		}
		if ( 'POST' === $request_method ) {
			$this->process_form( $id );
		}

		$icons_repository = new IconsRepository();
		$icon             = $icons_repository->get_by_id( new IconId( $id ) );
		if ( is_null( $icon ) ) {
			print '<div class="error"><p>' . esc_html__( 'Icon not found!', 'locrmaps-textdomain' ) . '</p></div>' . PHP_EOL;
			return;
		}

		print '<form method="post">';
		wp_nonce_field( 'edit_icon_item_nonce' );
		print '<table class="form-table">';
		print '<tbody>';
		print '<tr>';
		print '<td><label for="icon_id">' . esc_html__( 'ID', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td><input type="text" class="regular-text" id="icon_id" name="icon_id" readonly="readonly" value="' . esc_attr( (string) $id ) . '"/></td>';
		print '</tr>';
		print '<tr>';
		print '<td><label for="preview">' . esc_html__( 'Icon', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td><img src="' . esc_attr( (string) $icon->url ) . '" alt="preview" style="max-width: 36px; max-height: 36px;" id="preview" name="preview"/></td>';
		print '</tr>';
		print '<tr>';
		print '<td><label for="name">' . esc_html__( 'Name', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td><input type="text" class="regular-text" id="name" name="name" value="' . esc_attr( (string) $icon->name ) . '"/></td>';
		print '</tr>';
		print '<tr>';
		print '<td><label for="size">' . esc_html__( 'Size (in px)', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td><input type="text" class="regular-text" id="size" name="size" readonly="readonly" value="' . esc_attr( $icon->size->width . 'x' . $icon->size->height ) . '"/></td>';
		print '</tr>';
		print '<tr>';
		print '<td><label for="scale_percentage">' . esc_html__( 'Scale (in %)', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td><input type="number" min="1" step="1" max="1000" class="regular-text" id="scale_percentage" name="scale_percentage" value="' . ( floatval( $icon->scale->value ) * 100 ) . '"/></td>';
		print '</tr>';
		print '<tr>';
		print '<td><label for="anchor">' . esc_html__( 'Anchor', 'locrmaps-textdomain' ) . '</label></td>';
		print '<td>';
		print '<select id="anchor" name="anchor" class="regular-text">';
		foreach ( $icon->anchor->cases() as $anchor ) {
			print '<option value="' . esc_attr( $anchor->value ) . '"';
			if ( $icon->anchor->value === $anchor->value ) {
				print ' selected="selected"';
			}
			print '>' . esc_html( $anchor->value ) . '</option>';
		}
		print '</select>';
		print '</td>';
		print '</tr>';
		print '</tbody>';
		print '</table>';
		submit_button();
		print '</form>';
	}

	/**
	 * This function displays the icon URL column.
	 *
	 * @return array<mixed>
	 */
	public function get_bulk_actions(): array {
		return array(
			'bulk-delete' => __( 'Delete', 'default' ),
		);
	}

	/**
	 * This function initializes the icon list table.
	 *
	 * @return array<mixed> The columns.
	 */
	public function get_columns(): array {
		return array(
			'cb'       => '<input type="checkbox" />',
			'id'       => __( 'ID', 'locrmaps-textdomain' ),
			'icon_url' => __( 'Icon', 'locrmaps-textdomain' ),
			'name'     => __( 'Name', 'locrmaps-textdomain' ),
			'size'     => __( 'Size', 'locrmaps-textdomain' ),
			'scale'    => __( 'Scale', 'locrmaps-textdomain' ),
			'anchor'   => __( 'Anchor', 'locrmaps-textdomain' ),
		);
	}

	/**
	 * This function gets the data.
	 *
	 * @return TypedSequence The data.
	 */
	public function get_data(): TypedSequence {
		$icons_repository = new IconsRepository();
		return $icons_repository->get_all_icons();
	}

	/**
	 * This function handles the table actions.
	 */
	public function handle_table_actions(): void {
		$wp_nonce      = null;
		$nonce_request = filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, '_wpnonce', FILTER_DEFAULT );
		if ( is_string( $nonce_request ) ) {
			$wp_nonce = $nonce_request;
		}

		$action  = filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action', FILTER_DEFAULT );
		$action2 = filter_input( INPUT_GET, 'action2', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action2', FILTER_DEFAULT );

		if ( 'bulk-delete' === $action || 'bulk-delete' === $action2 ) {
			$admin_url = admin_url( 'admin.php?page=' . Icons::SUB_PAGE_NAME );
			if ( ! is_string( $wp_nonce ) ) {
				wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::NoNonce->value, $admin_url ) );
				exit;
			}

			$ids = filter_input( INPUT_GET, 'id', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY ) ?? filter_input( INPUT_POST, 'id', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
			if ( is_array( $ids ) ) {
				$nonce_action = 'bulk-delete';
				if ( isset( $this->_args['plural'] ) && is_string( $this->_args['plural'] ) ) {
					$nonce_action = 'bulk-' . $this->_args['plural'];
				}
				if ( ! wp_verify_nonce( $wp_nonce, $nonce_action ) ) {
					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
					exit;
				}

				global $wpdb;
				// @phpstan-ignore property.nonObject
				$table_prefix = $wpdb->prefix;
				if ( ! is_string( $table_prefix ) ) {
					$table_prefix = '';
				}
				$table_name = $table_prefix . Plugin::SLUG . '_icons';

				foreach ( $ids as $id ) {
					// @phpstan-ignore method.nonObject
					$deleted = $wpdb->delete(
						table: $table_name,
						where: array( 'id' => $id ),
						where_format: array( '%d' )
					);
					if ( ! $deleted ) {
						wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::IconsNotDeleted->value, $admin_url ) );
						exit;
					}
				}

				wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::IconsDeleted->value, $admin_url ) );
				exit;
			}
		}
	}

	/**
	 * This function prepares the items.
	 */
	public function prepare_items(): void {
		$this->handle_table_actions();

		$data     = $this->get_data();
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = array(
			'id'   => array( 'id', true ),
			'name' => array( 'name', false ),
		);

		$this->_column_headers = array( $columns, $hidden, $sortable );
		$this->items           = $data->to_array();
	}

	/**
	 * This function processes the form.
	 *
	 * @param int|string $id The id.
	 */
	private function process_form( $id ): void {
		$admin_url = admin_url( 'admin.php?page=' . Icons::SUB_PAGE_NAME );

		$wp_nonce = (string) ( filter_input( INPUT_POST, '_wpnonce', FILTER_DEFAULT ) ?? '' );

		if ( ! wp_verify_nonce( $wp_nonce, 'edit_icon_item_nonce' ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$icon_id = intval( filter_input( INPUT_GET, 'icon_id', FILTER_VALIDATE_INT ) ?? 0 );
		if ( ! $icon_id ) {
			print '<div class="error"><p>' . esc_html__( 'No "icon_id"!', 'locrmaps-textdomain' ) . '</p></div>' . PHP_EOL;
			return;
		}

		global $wpdb;

		$icon_name = (string) ( filter_input( INPUT_POST, 'name', FILTER_DEFAULT ) ?? '' );

		$icon_scale_percentage = filter_input( INPUT_POST, 'scale_percentage', FILTER_VALIDATE_INT );
		if ( ! is_numeric( $icon_scale_percentage ) ) {
			$icon_scale_percentage = 100;
		}

		$icon_anchor_string = filter_input( INPUT_POST, 'anchor', FILTER_DEFAULT );
		if ( ! is_string( $icon_anchor_string ) ) {
			$icon_anchor_string = Anchor::Center->value;
		}
		$icon_anchor = Anchor::from( $icon_anchor_string );

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		// @phpstan-ignore method.nonObject
		$updated = $wpdb->update(
			table: $table_prefix . 'locrmaps_icons',
			data: array(
				'name'   => sanitize_text_field( $icon_name ),
				'scale'  => $icon_scale_percentage / 100.0,
				'anchor' => $icon_anchor->value,
			),
			where: array( 'id' => $icon_id ),
		);

		if ( false !== $updated ) {
			print '<div class="updated">';
			print '<p>' . esc_html__( 'Icon updated!', 'locrmaps-textdomain' ) . '</p>';
			print '<p><a href="' . esc_url( $admin_url ) . '">← ' . esc_html__( 'Back to the icon list', 'locrmaps-textdomain' ) . '</a></p>';
			print '</div>' . PHP_EOL;
		} else {
			print '<div class="error"><p>' . esc_html__( 'Icon not updated!', 'locrmaps-textdomain' ) . '</p></div>' . PHP_EOL;
		}
	}
}
