<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Repositories;

use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap\Location;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Id as OnlineMapId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\Id as LocationId;
use Locr\WordPress\Plugin\LocrMaps\{Plugin, TypedSequence};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icons.
 */
final class OnlineMapLocations {
	/**
	 * This function adds an ONLINEmap into the database.
	 *
	 * @param OnlineMapId $onlinemap_id The ONLINEmap id.
	 * @param Location    $location The ONLINEmap to add.
	 * @throws \Exception If the ONLINEmap could not be added.
	 */
	public function add( OnlineMapId $onlinemap_id, Location $location ): void {
		global $wpdb;

		$insert_data   = array(
			'onlinemap_id' => $onlinemap_id->value,
			'icon_id'      => $location->icon_id->value,
			'latitude'     => $location->position->latitude,
			'longitude'    => $location->position->longitude,
			'address'      => $location->address->to_json(),
		);
		$insert_format = array(
			'%d',
			'%d',
			'%f',
			'%f',
			'%s',
		);

		// @phpstan-ignore method.nonObject
		$insert_result = $wpdb->insert(
			table: $this->get_tablename(),
			data: $insert_data,
			format: $insert_format
		);

		if ( false === $insert_result ) {
			throw new \Exception( 'The ONLINEmap could not be added (db-error).' );
		}

		// @phpstan-ignore property.nonObject
		$location_id = new LocationId( $wpdb->insert_id );
		$location->update_id( $location_id );
	}

	/**
	 * This function deletes an ONLINEmap by id.
	 *
	 * @param LocationId $id The id of the ONLINEmap.
	 * @throws \Exception If the ONLINEmap could not be deleted.
	 */
	public function delete_by_id( LocationId $id ): void {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$deleted = $wpdb->delete(
			table: $this->get_tablename(),
			where: array( 'id' => $id->value ),
			where_format: array( '%d' )
		);
		if ( false === $deleted ) {
			throw new \Exception( 'The ONLINEmap could not be deleted (db-error).' );
		}
	}

	/**
	 * This function deletes all icons by ids.
	 *
	 * @param array<mixed> $ids The ids of the icons.
	 * @param OnlineMapId  $onlinemap_id The id of the ONLINEmap.
	 * @throws \Exception If the icons could not be deleted.
	 */
	public function delete_all_ids( array $ids, OnlineMapId $onlinemap_id ): void {
		global $wpdb;

		foreach ( $ids as $id ) {
			// @phpstan-ignore method.nonObject
			$deleted = $wpdb->delete(
				table: $this->get_tablename(),
				where: array(
					'id'           => $id,
					'onlinemap_id' => $onlinemap_id->value,
				),
				where_format: array( '%d', '%d' )
			);
			if ( false === $deleted ) {
				throw new \Exception( 'The ONLINEmap locations could not be deleted at all (db-error).' );
			}
		}
	}

	/**
	 * This function gets an ONLINEmap location by id.
	 *
	 * @param OnlineMapId $onlinemap_id The id of the ONLINEmap location.
	 */
	public function get_by_id( OnlineMapId $onlinemap_id ): TypedSequence {
		global $wpdb;

		$tablename = $this->get_tablename();
		// @phpstan-ignore method.nonObject
		$db_result = $wpdb->get_results(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				"SELECT * FROM {$tablename} WHERE onlinemap_id = %s", // phpcs:ignore
				$onlinemap_id->value
			),
			ARRAY_A
		);

		$locations = TypedSequence::for_type( Location::class );

		if ( is_array( $db_result ) ) {
			foreach ( $db_result as $row ) {
				$locations->add( Location::from_db_row( $row ) );
			}
		}

		return $locations;
	}

	/**
	 * This function gets the table name.
	 */
	public function get_tablename(): string {
		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		return $table_prefix . Plugin::SLUG . '_onlinemap_locations';
	}

	/**
	 * This function updates an ONLINEmap location.
	 *
	 * @param Location $location The ONLINEmap to update.
	 * @throws \Exception If the ONLINEmap could not be updated.
	 */
	public function update( Location $location ): void {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$updated = $wpdb->update(
			table: $this->get_tablename(),
			data: array(
				'icon_id'   => $location->icon_id->value,
				'latitude'  => $location->position->latitude,
				'longitude' => $location->position->longitude,
				'address'   => $location->address->to_json(),
			),
			where: array( 'id' => $location->id->value ),
		);

		if ( false === $updated ) {
			throw new \Exception( 'The ONLINEmap location could not be updated (db-error).' );
		}
	}
}
